package tracker;
import tracker.TrackerManagementTCP;
import tracker.TrackerManagementUDP;
import tools.Directories;
import tools.Logger;

/** Tracker implementation
 * First argument of main method is port listened by the tracker, and is mandatory.
 * @author	Louis Royer
 * @author	Flavien Haas
 * @author	JS Auge
 * @version	1.0
 */
public class Tracker {
	private int port;
	private Directories directories;
	private Logger logger;

	/** Constructor with portStr containing a port number.
	 * @param portStr	String containing port number of listening.
	 */
	public Tracker(String portStr) {
		port = Integer.valueOf(Integer.parseInt(portStr));
		directories = new Directories("P2P_JAVA_PROJECT_TRACKER_" + port);
		logger = new Logger(directories.getDataHomeDirectory() + "tracker.log");
		System.out.println("Tracker will listen on port " + port + " and write logs into " + directories.getDataHomeDirectory());
		directories.askOpenDataHomeDirectory(null);
	}

	/** Main program entry point
	 * first parameter is port number and is mandatory
	 * to test, run with: java -ea serverP2P.ServerP2P -- <portNumber>
	 * @param args	parameters
	 */
		public static void main(String [] args) {
		Tracker t = new Tracker(args[1]);
		TrackerManagementUDP tmudp = new TrackerManagementUDP(t.port, t.logger);
		TrackerManagementTCP tmtcp = new TrackerManagementTCP(t.port, t.logger);
		Thread tudp = new Thread(tmudp);
		tudp.setName("Tracker UDP P2P-JAVA-PROJECT");
		tudp.start();
		Thread ttcp = new Thread(tmtcp);
		ttcp.setName("Tracker TCP P2P-JAVA-PROJECT");
		ttcp.start();
	}

}
